@@ -40,13 +40,14 @@
 #' 
 #' @export
 #Degree----
+# Updated 18.05.2020
 degree <- function (A)
 {
     if(nrow(A)!=ncol(A))
     {stop("Input not an adjacency matrix")}
     
-    A <- abs(A)
     A <- as.matrix(A)
+    
     A <- binarize(A)
     
     if(isSymmetric(A, check.attributes = FALSE))
@@ -57,22 +58,36 @@ degree <- function (A)
     }else
     {
         #In-degree
-        inDeg <- as.vector(colSums(A))
+        inDeg <- as.vector(colSums(A, na.rm = TRUE))
         names(inDeg) <- colnames(A)
         
         #Out-degree
-        outDeg <- as.vector(rowSums(A))
+        outDeg <- as.vector(rowSums(A, na.rm = TRUE))
         names(outDeg) <- colnames(A)
         
         #Relative influence
         relinf <- as.vector((outDeg-inDeg)/(outDeg+inDeg))
         names(relinf) <- colnames(A)
+        
+        #Reciprocal degree
+        for(i in 1:nrow(A))
+            for(j in 1:ncol(A))
+            {
+                A[i,j] <- ifelse(A[i,j] == 1 & A[j,i] == 1, 1, 0)
+                A[j,i] <- ifelse(A[i,j] == 1 & A[j,i] == 1, 1, 0)
+            }
+        
+        recipDeg <- colSums(A, na.rm = TRUE)
+        names(recipDeg) <- colnames(A)
     
     if(all(relinf<.001))
     {Deg <- as.vector(inDeg)
     names(Deg) <- colnames(A)
     return(Deg)
-    }else{return(list(inDegree=inDeg,outDegree=outDeg,relInf=relinf))}
+    }else{return(list(inDegree=inDeg,
+                      outDegree=outDeg,
+                      recipDegree=recipDeg,
+                      relInf=relinf))}
     }
 }
 #----
\ No newline at end of file
